@@ -7,7 +7,7 @@ use Pcore::Util::Scalar qw[is_plain_coderef is_plain_scalarref];
 
 with qw[Pcore::Util::Result::Status];
 
-has url => ( is => 'ro', isa => Str | Object, writer => 'set_url' );
+has url => ( is => 'ro', isa => Str | Object );
 has buf_size => ( is => 'ro', isa => PositiveOrZeroInt, default => 0 );    # write body to fh if body length > this value, 0 - always store in memory, 1 - always store to file
 
 has version => ( is => 'ro', isa => Num, init_arg => undef );
@@ -17,7 +17,7 @@ has path => ( is => 'ro', isa => Str, init_arg => undef );
 
 has content_length => ( is => 'rwp', isa => PositiveOrZeroInt, default => 0, init_arg => undef );
 
-has redirect => ( is => 'lazy', isa => ArrayRef, default => sub { [] }, init_arg => undef );
+has redirect => ( is => 'ro', isa => ArrayRef, init_arg => undef );
 has decoded_body => ( is => 'lazy', isa => Maybe [ScalarRef], init_arg => undef );
 has is_connect_error => ( is => 'ro', isa => Bool, default => 0, init_arg => undef );
 
